.gfk-related-widget-wrapper {
  max-width: 520px;
  float: none;
}

.gfk-related-widget-wrapper .select2-container--admin-autocomplete.select2-container {
  width: 45% !important;
}

.generic-foreign-key-fk-field {
  display: inline-block;
  max-width: 260px;
  width: 45%;
}

.generic-foreign-key-fk-field:empty {
  width: auto;
}

.generic-foreign-key-fk-field:empty + a + a,
.generic-foreign-key-fk-field:empty + a {
  display: none;
}

.generic-foreign-key-fk-field .select2-container--admin-autocomplete.select2-container {
  width: 100% !important;
}

.clear-generic-field,
.edit-generic-field {
  width: 3%;
  margin-left: 3px;
}

.clear-generic-field img {
    width: 13px;
    height: 13px;
}
