from .permissions_readonly import PermissionSelectMultipleReadonlyWidget
from .foreign_key import ForeignKeyReadonlyWidget
from .many_to_many import ManyToManyReadonlyWidget
from .generic_foreign_key import GenericForeignKeyReadonlyWidget
from .string_field import StringReadonlyWidget
from .boolean import BooleanReadonlyWidget
